' -------------------------------------------------------------------------
'       From the book Inside Active Directory, ISBN 0-201-61621-1
'       Copyright (C) 2002 by Addison-Wesley
'       Script by Sakari Kouti (see http://www.kouti.com)
' You have a royalty-free right to use, modify, reproduce and distribute
' this script (and/or any modified version) in any way you find useful,
' provided that you agree that Addison-Wesley or Sakari Kouti has no
' warranty, obligations or liability for the script. If you modify
' the script, you must retain this copyright notice.
' -------------------------------------------------------------------------
Option Explicit

Const UF_ACCOUNTDISABLE					= &H0002
Const UF_PASSWD_NOTREQD					= &H0020
Const UF_WORKSTATION_TRUST_ACCOUNT		= &H1000 'ws or ms comp

Const ADS_RIGHT_DS_SELF					= &H8
Const ADS_RIGHT_DS_WRITE_PROP			= &H20
Const ADS_RIGHT_DS_CONTROL_ACCESS		= &H100
Const ADS_ACETYPE_ACCESS_ALLOWED_OBJECT = &H5
Const ADS_FLAG_OBJECT_TYPE_PRESENT		= &H1

Const GUID_ACCOUNT_RESTRICTIONS			= "{4C164200-20C0-11D0-A768-00AA006E0529}"
Const GUID_VALIDATED_SPN				= "{F3A64788-5306-11D1-A9C5-0000F80367C1}"
Const GUID_VALIDATED_DNSHOST			= "{72E39547-7B18-11D1-ADEF-00C04FD8D5CD}"
Const GUID_RESET_PASSWORD				= "{00299570-246D-11D0-A768-00AA006E0529}"

Call CreateComputer("PC17", "Red.Baron@sanao.com")

'===================================

Sub CreateComputer(strComputerName, strWhoCanJoin)
	Dim objDSE, objComputersContainer, objComputer, objSecDesc
	Dim objDACL, objACE1, objACE2, objACE3, objACE4

	Set objDSE = GetObject("LDAP://rootDSE")
	Set objComputersContainer = GetObject("LDAP://CN=Computers," & _
		objDSE.Get("defaultNamingContext"))

	Set objComputer = objComputersContainer.Create("computer", _
		"CN=" & strComputerName)
	Call objComputer.Put("sAMAccountName", strComputerName & "$")
	Call objComputer.Put("userAccountControl", _
		UF_PASSWD_NOTREQD _
		Or UF_WORKSTATION_TRUST_ACCOUNT _
		) 'Or UF_ACCOUNTDISABLE   We want to enable, not disable

	objComputer.SetInfo

	Set objACE1 = CreateObject("AccessControlEntry")
	objACE1.Trustee		= strWhoCanJoin
	objACE1.AccessMask	= ADS_RIGHT_DS_WRITE_PROP
	objACE1.AceFlags	= 0	'no inheritance from up or to below
	objACE1.AceType		= ADS_ACETYPE_ACCESS_ALLOWED_OBJECT
	objACE1.Flags		= ADS_FLAG_OBJECT_TYPE_PRESENT
	objACE1.ObjectType	= GUID_ACCOUNT_RESTRICTIONS
	'objACE1.InheritedObjectType = not used

	Set objACE2 = CreateObject("AccessControlEntry")
	objACE2.Trustee		= strWhoCanJoin
	objACE2.AccessMask	= ADS_RIGHT_DS_SELF
	objACE2.AceFlags	= 0	'no inheritance from up or to below
	objACE2.AceType		= ADS_ACETYPE_ACCESS_ALLOWED_OBJECT
	objACE2.Flags		= ADS_FLAG_OBJECT_TYPE_PRESENT
	objACE2.ObjectType	= GUID_VALIDATED_SPN
	'objACE2.InheritedObjectType = not used

	Set objACE3 = CreateObject("AccessControlEntry")
	objACE3.Trustee		= strWhoCanJoin
	objACE3.AccessMask	= ADS_RIGHT_DS_SELF
	objACE3.AceFlags	= 0	'no inheritance from up or to below
	objACE3.AceType		= ADS_ACETYPE_ACCESS_ALLOWED_OBJECT
	objACE3.Flags		= ADS_FLAG_OBJECT_TYPE_PRESENT
	objACE3.ObjectType	= GUID_VALIDATED_DNSHOST
	'objACE3.InheritedObjectType = not used

	Set objACE4 = CreateObject("AccessControlEntry")
	objACE4.Trustee		= strWhoCanJoin
	objACE4.AccessMask	= ADS_RIGHT_DS_CONTROL_ACCESS
	objACE4.AceFlags	= 0	'no inheritance from up or to below
	objACE4.AceType		= ADS_ACETYPE_ACCESS_ALLOWED_OBJECT
	objACE4.Flags		= ADS_FLAG_OBJECT_TYPE_PRESENT
	objACE4.ObjectType	= GUID_RESET_PASSWORD
	'objACE4.InheritedObjectType = not used

	Set objSecDesc = objComputer.Get("ntSecurityDescriptor")
	Set objDACL = objSecDesc.DiscretionaryAcl

	Call objDACL.AddAce(objACE1)
	Call objDACL.AddAce(objACE2)
	Call objDACL.AddAce(objACE3)
	Call objDACL.AddAce(objACE4)

	objSecDesc.DiscretionaryAcl = objDACL
	Call objComputer.Put("ntSecurityDescriptor", objSecDesc)
	objComputer.SetInfo

	WScript.Echo "Created computer " & strComputerName
End Sub
